@@ -118,27 +118,28 @@ class ObjectNode(Node):
 
 class BaseAdapter:
     """Handles serialisation of a specific object type"""
-    def pack(self, obj, context):
+    def build_node(self, obj, context):
         """
-        Translates obj into serialisable form. Any media declarations that will be required for
-        deserialisation of the object should be passed to context.add_media().
+        Translates obj into a node that we can call emit() on to obtain the final serialisable
+        form. Any media declarations that will be required for deserialisation of the object should
+        be passed to context.add_media().
 
-        This base implementation handles simple JSON-serialisable values such as strings, and
-        returns them unchanged.
+        This base implementation handles simple JSON-serialisable values such as integers, and
+        wraps them as a ValueNode.
         """
         return ValueNode(obj)
 
 
 class StringAdapter(BaseAdapter):
-    def pack(self, obj, context):
+    def build_node(self, obj, context):
         return StringNode(obj)
 
 
 class DictAdapter(BaseAdapter):
     """Handles serialisation of dicts"""
-    def pack(self, obj, context):
+    def build_node(self, obj, context):
         return DictNode({
-            str(key): context.pack(val)
+            str(key): context.build_node(val)
             for key, val in obj.items()
         })
 
@@ -160,12 +161,23 @@ class Adapter(BaseAdapter, metaclass=MediaDefiningClass):
 
     def pack(self, obj, context):
         context.add_media(self.get_media(obj))
+        return (self.js_constructor, self.js_args(obj))
+
+    def build_node(self, obj, context):
+        constructor, args = self.pack(obj, context)
         return ObjectNode(
-            self.js_constructor,
-            [context.pack(arg) for arg in self.js_args(obj)]
+            constructor, [context.build_node(arg) for arg in args]
         )
 
 
+class AutoAdapter(Adapter):
+    """
+    Adapter for objects that define their own telepath_pack method that we can simply delegate to.
+    """
+    def pack(self, obj, context):
+        return obj.telepath_pack(context)
+
+
 class JSContextBase:
     """
     Base class for JSContext classes obtained through AdapterRegistry.js_context_class.
@@ -188,14 +200,24 @@ class JSContextBase:
     def base_media(self):
         return forms.Media(js=[self.telepath_js_path])
 
-    def add_media(self, media):
-        media_str = str(media)
-        if media_str not in self.media_fragments:
-            self.media += media
-            self.media_fragments.add(media_str)
+    def add_media(self, media=None, js=None, css=None):
+        media_objects = []
+        if media:
+            media_objects.append(media)
+        if js or css:
+            if isinstance(js, str):
+                # allow passing a single JS file name as equivalent to a singleton list
+                js = [js]
+            media_objects.append(forms.Media(js=js, css=css))
+
+        for media_obj in media_objects:
+            media_str = str(media_obj)
+            if media_str not in self.media_fragments:
+                self.media += media_obj
+                self.media_fragments.add(media_str)
 
     def pack(self, obj):
-        return ValueContext(self).pack(obj).emit()
+        return ValueContext(self).build_node(obj).emit()
 
 
 class AdapterRegistry:
@@ -220,8 +242,42 @@ class AdapterRegistry:
             # superclass, so will be handled as a special case
         }
 
-    def register(self, adapter, cls):
-        self.adapters[cls] = adapter
+    def register(self, *args, **kwargs):
+        if len(args) == 2 and not kwargs:
+            # called as register(adapter, cls)
+            adapter, cls = args
+            if not isinstance(adapter, Adapter):
+                raise TypeError("register expected an Adapter instance, got %r" % adapter)
+
+            self.adapters[cls] = adapter
+
+        elif not args:
+            # called as a class decorator: @register() or @register(adapter=MyAdapter()) -
+            # the return value here is the function that will receive the class definition
+            adapter = kwargs.get('adapter') or AutoAdapter()
+            if not isinstance(adapter, Adapter):
+                raise TypeError("register expected an Adapter instance, got %r" % adapter)
+
+            def wrapper(cls):
+                # register the class and return it unchanged
+                self.adapters[cls] = adapter
+                return cls
+
+            return wrapper
+
+        elif len(args) == 1 and isinstance(args[0], type):
+            # called as a class decorator @register without parentheses -
+            # we are passed the class definition here
+            cls = args[0]
+            self.adapters[cls] = AutoAdapter()
+            return cls
+
+        else:
+            raise TypeError(
+                "register must be called as register(adapter, cls) or as a class decorator - "
+                "@register or @register(adapter=MyAdapter())"
+            )
+
 
     def find_adapter(self, cls):
         for base in cls.__mro__:
@@ -239,48 +295,48 @@ class AdapterRegistry:
 
 class ValueContext:
     """
-    A context instantiated for each top-level value that JSContext.pack is called on.
-    Values packed in this context will be kept in a lookup table; if over the course of
-    packing the top level value we encounter multiple references to the same value, a
-    reference to the previously-packed value will be generated rather than packing it
-    again. Calls to add_media are passed back to the parent context so that multiple
-    calls to pack() will have their media combined in a single bundle.
+    A context instantiated for each top-level value that JSContext.pack is called on. Results from
+    this context's build_node method will be kept in a lookup table. If, over the course of
+    building the node tree for the top level value, we encounter multiple references to the same
+    value, a reference to the existing node will be generated rather than building it again. Calls
+    to add_media are passed back to the parent context so that multiple calls to pack() will have
+    their media combined in a single bundle.
     """
     def __init__(self, parent_context):
         self.parent_context = parent_context
         self.registry = parent_context.registry
         self.raw_values = {}
-        self.packed_values = {}
+        self.nodes = {}
         self.next_id = 0
 
-    def add_media(self, media):
-        self.parent_context.add_media(media)
+    def add_media(self, *args, **kwargs):
+        self.parent_context.add_media(*args, **kwargs)
 
-    def pack(self, val):
+    def build_node(self, val):
         obj_id = id(val)
         try:
-            existing_packed_val = self.packed_values[obj_id]
+            existing_node = self.nodes[obj_id]
         except KeyError:
-            # not seen this value before, so pack it and store in packed_values
-            packed_val = self._pack_as_value(val)
-            self.packed_values[obj_id] = packed_val
+            # not seen this value before, so build a new node for it and store in self.nodes
+            node = self._build_new_node(val)
+            self.nodes[obj_id] = node
             # Also keep a reference to the original value to stop it from getting deallocated
             # and the ID being recycled
             self.raw_values[obj_id] = val
 
-            return packed_val
+            return node
 
-        if existing_packed_val.id is None:
-            # Assign existing_packed_val an ID so that we can create references to it
-            existing_packed_val.id = self.next_id
+        if existing_node.id is None:
+            # Assign existing_node an ID so that we can create references to it
+            existing_node.id = self.next_id
             self.next_id += 1
 
-        return existing_packed_val
+        return existing_node
 
-    def _pack_as_value(self, obj):
+    def _build_new_node(self, obj):
         adapter = self.registry.find_adapter(type(obj))
         if adapter:
-            return adapter.pack(obj, self)
+            return adapter.build_node(obj, self)
 
         # No adapter found; try special-case fallbacks
 
@@ -295,7 +351,7 @@ class ValueContext:
         except TypeError:  # obj is not iterable
             raise UnpackableTypeError("don't know how to pack object: %r" % obj)
         else:
-            return ListNode([self.pack(item) for item in items])
+            return ListNode([self.build_node(item) for item in items])
 
 
 # define a default registry of adapters. Typically this will be the only instance of
@@ -304,7 +360,4 @@ class ValueContext:
 
 registry = AdapterRegistry()
 JSContext = registry.js_context_class
-
-
-def register(adapter, cls):
-    registry.register(adapter, cls)
+register = registry.register
