@@ -11,11 +11,17 @@ class Artist:
         self.name = name
 
 
+@register
 class Album:
     def __init__(self, title, artists):
         self.title = title
         self.artists = artists
 
+    def telepath_pack(self, context):
+        context.add_media(js='music_player.js')
+
+        return ('music.Album', [self.title, self.artists])
+
 
 class ArtistAdapter(Adapter):
     js_constructor = 'music.Artist'
@@ -27,19 +33,6 @@ class ArtistAdapter(Adapter):
 register(ArtistAdapter(), Artist)
 
 
-class AlbumAdapter(Adapter):
-    js_constructor = 'music.Album'
-
-    def js_args(self, obj):
-        return [obj.title, obj.artists]
-
-    class Media:
-        js = ['music_player.js']
-
-
-register(AlbumAdapter(), Album)
-
-
 class TestPacking(TestCase):
     def test_pack_object(self):
         beyonce = Artist("Beyoncé")
